# ---------------------------------------------- Whole Life Civil Systems Analysis WiSe 22/23 -------------------------------------
# ---------------------------------------------- Group D --------------------------------------------------------------------------
#Asin Akbulut 
#Muhammed Faruk Aydin
#Juan Felipe De Queiroz Leles
#Pengfei Guo
#Yekta Tektas

#----------------------------------------------- R Script Begin -------------------------------------------------------------------
rm (list = ls())
#Initialization
library(timelineS) 
library(dplyr) 
library(MCDA) 
library(ggplot2) 
library(scales) 
library(reshape2) 
library(tibble)
library(lubridate)
library(rPref)
library(mco)
library(MASS)

# ---------------------------------------------- Part I - Integrated Maintenance Planning ---------------------------------------

plot.timeline <- function(lifetime, events.name, start.Date, plot.Name) {
  dataP <- data.frame(Events = events.name, Event_Dates = ymd(start.Date) + 
                        years(lifetime))
  timelineS(dataP, main = plot.Name,labels = events.name, label.direction = "up", label.position = 3)
}
dist.Events <- function (lifetime, events, start.Date, option.Name) {

  events <- sort(events, decreasing = T)
  distribution.events <- sapply(events, seq, from = 0, to = lifetime)
  all.events <- melt(distribution.events)
  colnames(all.events) <- c("frequency", "event")
  all.events <- all.events[order(all.events$frequency),]
  unique.events <- all.events[!duplicated(all.events$frequency),]
  unique.events$event[which(unique.events$frequency == 0)] <- "DC"
  #plot.timeline(unique.events$frequency, unique.events$event, start.Date, option.Name)
  return(unique.events)
}


combine.lifeTimelines <- function(product.1, product.2, p1.dur, p2.dur) {
  
  base <- list(Names = c(), frequency = c(), duration = c())
  
  base$frequency <- sort(unique(c(product.1$frequency,product.2$frequency)),decreasing = FALSE)
  base$Names[1] <- "DC"
  base$duration[1] <- 0
  base$Names[length(base$frequency)] <- "END"
  
  base$duration[length(base$frequency)] <- 0
  for(index in 2:(length(base$frequency) - 1)) {
    phase.1 <- product.1$event[which(product.1$frequency == base$frequency[index])]
    phase.2 <- product.2$event[which(product.2$frequency == base$frequency[index])]
    if(length(phase.1) != 0 && length(phase.2) != 0) {
      base$Names[index] <- paste(phase.1, phase.2)
      base$duration[index] <- max(p1.dur[which(names(p1.dur) == phase.1)],
                                  p2.dur[which(names(p2.dur) == phase.2)])
    } else if(length(phase.1) != 0 && length(phase.2) == 0) {
      base$Names[index] <- phase.1
      base$duration[index] <- p1.dur[which(names(p1.dur) == phase.1)]
    } else if (length(phase.1) == 0 && length(phase.2) != 0) {
      base$Names[index] <- phase.2
      base$duration[index] <- p2.dur[which(names(p2.dur) == phase.2)]
    } else {
      base$duration[index] <- 0
    }
  }
  return(base)
}

par(mfrow = c(2, 1)) #split the plot in 2 rows and 1 column
start.Date = "2020-01-01"
lifetime <- 60

#First strategy of maintenance:
events.Op1.Kindergarten <- c(M1 = 5, M2 = 20, M3 = 40, END = lifetime) #interval between interventions in Years
duration.ev.Kindergarten <- c(M1 = 1, M2 = 2, M3 = 10) #duration of interventions in Days
events.Op1.Exterior <- c(E1 = 2, E2 = 10, E3 = 20, END = lifetime) 
duration.ev.Exterior <- c(E1 = 1, E2 = 2, E3 = 30)

design.Options.Kindergarten <- list(design.Op1.Kindergarten <- "Main-Structure - Kindergarden")
design.Options.Exterior <- list(design.Op1.Exterior <- "Exterior")

maintenance.Kindergarten <- dist.Events(lifetime, events.Op1.Kindergarten, start.Date, design.Options.Kindergarten)
maintenance.Exterior <- dist.Events(lifetime, events.Op1.Exterior, start.Date, design.Options.Exterior)
plot.timeline(maintenance.Kindergarten$frequency, maintenance.Kindergarten$event, start.Date, design.Options.Kindergarten)
plot.timeline(maintenance.Exterior$frequency, maintenance.Exterior$event, start.Date, design.Options.Exterior)

integrated.interv <- combine.lifeTimelines(maintenance.Kindergarten, maintenance.Exterior, duration.ev.Kindergarten, duration.ev.Exterior)
sum(integrated.interv$duration) # total duration of the interruptions for this maintenance strategy

#Second strategy of maintenance: 
events.Op1.Kindergarten = c(M1 = 5, M2 = 20, M3 = 40, END = lifetime)
duration.ev.Kindergarten <- c(M1 = 2, M2 = 1, M3 = 10)

events.Op1.Exterior <- c(E1 = 2, E2 = 100, E3 = 40, END = lifetime)
duration.ev.Exterior <- c(E1 = 3, E2 = 0, E3 = 10)

design.Options.Kindergarten <- list(design.Op1.Kindergarten <- "Main-Structure")
design.Options.Exterior <- list(design.Op1.Exterior <- "Exterior")

maintenance.Kindergarten <- dist.Events(lifetime, events.Op1.Kindergarten, start.Date, design.Options.Kindergarten)
maintenance.Exterior <- dist.Events(lifetime, events.Op1.Exterior, start.Date, design.Options.Exterior)

integrated.interv <- combine.lifeTimelines(maintenance.Kindergarten, maintenance.Exterior, duration.ev.Kindergarten, duration.ev.Exterior)
sum(integrated.interv$duration) # total duration of the interruptions for this maintenance strategy

getmode <- function(v) {
  uniqv <- unique(v)
  uniqv[which.max(tabulate(match(v, uniqv)))]
}

#Function to automate the search for the best maintenance strategy 
design.explore <- function(events1, events2) {
  results <- c()
  for(i in 1: dim(events1)[1]) {
    ev1 <- unlist(events1[i, ])
    dist.1 <- dist.Events(lifetime, ev1, start.Date, design.Op1.Kindergarten)
    dur.ev1 <- ev1/2
    for (j in 1: dim(events1)[1]) {
      ev2 <- unlist(events2[j, ])
      dist.2 <- dist.Events(lifetime, ev2, start.Date, design.Op1.Exterior)
      dur.ev2 <- ev2/2
      combined.lifetime <- combine.lifeTimelines(dist.1, dist.2, dur.ev1, dur.ev2)
      min.dist.int <- min(abs(combined.lifetime$frequency[1:(length(combined.lifetime$frequency) - 1)] -
                                combined.lifetime$frequency[2:length(combined.lifetime$frequency)]))
      results <- rbind(results, c(ev1, ev2,  dur = sum(combined.lifetime$duration), dist.inter = min.dist.int))
    }
  }
  return(as.data.frame(results))
}

#Generating the scenarios matrix (event1 and event2 in the function design.explore)
n.grid <- 5
events.grid.kindergarten <- expand.grid(M1 = sample(seq(10,20, by = 1), n.grid),
                                        M2 = sample(seq(3,8,1), n.grid),
                                        M3 = sample(seq(20, 35, 1), n.grid))
events.grid.exterior <- expand.grid(E1.r = sample(seq(12, 25, by = 1), n.grid),
                                  E2.r = sample(seq(5,10,1), n.grid),
                                  E3.r = sample(seq(27, 37, 1), n.grid))
response.space <- design.explore(events.grid.kindergarten, events.grid.exterior)

p <- low(dur)* high(dist.inter)
sky <- psel(response.space, p)
sky
#      M1 M2 M3 E1.r E2.r E3.r  dur dist.inter
#2375  19  7 35   19    7   35 61.0          1
#2856  10  8 35   16    8   35 78.5          2
#2866  10  8 35   16   10   35 78.5          2
#12431 12  8 28   16    8   32 86.0          4
pareto2 <- psel(response.space, p, top = nrow(response.space))
ggplot(response.space, aes(x = dur, y = dist.inter)) +
  geom_point(shape = 21) +
  geom_point(data = pareto2, size = 3, aes(color = factor(pareto2$.level)))
show_front <- function(pref) {
  plot(response.space$dur, response.space$dist.inter)
  sky <- psel(response.space, pref)
  plot_front(response.space, pref, col = rgb(0, 0, 1))
  points(sky$dur, sky$dist.inter, lwd = 3)
}
show_front(p)
p <- high(dur) * low(dist.inter)
show_front(p)


#----------------------------------------------- Part II - Integrated LCA and LCI ------------------------------------------------

LCI.materials <- read.csv("LCICombine.csv")
#---------------------------------------------------- LCA Foundation -------------------------------------------------------------


#Parameters for Foundation
Depth.pile<-36 
Depth.footing<-18
Depth.raft<-2.4
Depth.FCC<-1.2
Factor.pile.rcc<-0.025
Factor.footing.rcc<-0.03
Factor.footing.FCC<-0.1

LCA.Foundation<-function(length,width,design.Op,material){
  
  
  RCC.volume<-length*width*Depth.footing*Factor.footing.rcc
  intervention<-2 
  
  RCC.energy<-LCI.materials$energy[6]*RCC.volume*intervention+LCI.materials$energy[7]*RCC.volume*intervention
  +LCI.materials$energy[8]*RCC.volume*intervention+LCI.materials$energy[9]*RCC.volume*intervention
  RCC.CO2<-LCI.materials$CO2[6]*RCC.volume*intervention+LCI.materials$CO2[7]*RCC.volume*intervention
  +LCI.materials$CO2[8]*RCC.volume*intervention+LCI.materials$CO2[9]*RCC.volume*intervention
  RCC.NOx<-LCI.materials$NOx[6]*RCC.volume*intervention+LCI.materials$NOx[7]*RCC.volume*intervention
  +LCI.materials$NOx[8]*RCC.volume*intervention+LCI.materials$NOx[9]*RCC.volume*intervention
  RCC.SO2<-LCI.materials$SO2[6]*RCC.volume*intervention+LCI.materials$SO2[7]*RCC.volume*intervention
  +LCI.materials$SO2[8]*RCC.volume*intervention+LCI.materials$SO2[9]*RCC.volume*intervention
  
  
  FCC.volume<-length*width*Depth.FCC*Factor.footing.FCC
  intervention.FCC<-2 
  
  FCC.energy<-LCI.materials$energy[10]*FCC.volume*intervention.FCC+LCI.materials$energy[11]*FCC.volume*intervention.FCC
  +LCI.materials$energy[12]*FCC.volume*intervention.FCC
  FCC.CO2<-LCI.materials$CO2[10]*FCC.volume*intervention.FCC+LCI.materials$CO2[11]*FCC.volume*intervention.FCC
  +LCI.materials$CO2[12]*FCC.volume*intervention.FCC
  FCC.NOx<-LCI.materials$NOx[10]*FCC.volume*intervention.FCC+LCI.materials$NOx[11]*FCC.volume*intervention.FCC
  +LCI.materials$NOx[12]*FCC.volume*intervention.FCC
  FCC.SO2<-LCI.materials$SO2[10]*FCC.volume*intervention.FCC+LCI.materials$SO2[11]*FCC.volume*intervention.FCC
  +LCI.materials$SO2[12]*FCC.volume*intervention.FCC
  
  LCA.results <- list(Energy = (RCC.energy+FCC.energy) ,   
                      CO2 = (RCC.CO2+FCC.CO2),         
                      NOx = (RCC.NOx+FCC.NOx),
                      SO2 = (RCC.SO2+FCC.SO2))
  
  return(LCA.results)
  
}

#Initialize the variables

length=15
width=10

design.Op<-c("1. Isolated Footing","2. Pile Foundation","3. Raft Foundation")

Foundation.Option<-LCA.Foundation(length,width,design.Op = "1. Isolated Footing") #Best Design Option for 2nd Assignment

#---------------------------------------------------- LCA Structural Frame -------------------------------------------------------

#Parameters for Structural Frame
timber.frame.length <- 4 #length in m for squared timber profile
timber.frame.height <- 0.5 #h = width/25

LCA.kindergarten <- function(length, width, height, frame.Option, materials) {

    #volume
    frame.volume = timber.frame.length * width * timber.frame.height
    intervention <- 2
    
    #total consumption/emission 
    frame.energy<-LCI.materials$energy[21]*frame.volume*intervention
    frame.CO2<-LCI.materials$CO2[21]*frame.volume*intervention
    frame.NOx<-LCI.materials$NOx[21]*frame.volume*intervention
    frame.SO2<-LCI.materials$SO2[21]*frame.volume*intervention
  
  ##creation of list with added up energy consumption and emission
  LCA.results <- list(Energy = (frame.energy) ,   
                      CO2 = (frame.CO2),         
                      NOx = (frame.NOx),
                      SO2 = (frame.SO2))
  
  return(LCA.results)
  
}

#Initialize the variables

c.depth <- 0.3 #column depth in m
b.length <- 12 #beam length = frame width in m
k.length <- 17 #kindergarten length in m

frame.Options <- c("Timber", "Steel", "Reinforced Concrete")

Frame.Option <- LCA.kindergarten(c.depth, b.length, k.length, frame.Options[1], LCI.materials) #Best Design Option for 2nd Assignment

#---------------------------------------------------- LCA Bridge rail--------------------------------------------------------------

#Parameters for Rail
PRC.post.Section <- 0.12
timber.rails.Section<-0.08 #squared meters

LCA.bridgerail <- function(length, height, post.Option, rail.Option, materials, dist.Events) {
  
  materials.split <- split(materials, materials$scope)
  #interventions
  s <- summary(as.factor(dist.Events$event))
  s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors=F, row.names = 1:length(s))
  
  #post options
    n <- round(length / 1.1, 0)
    posts.V <- n * PRC.post.Section * 0.2
    interventions.post <- s2$DC + s2$E1 + 0.25*(if (!is.null(s2$E2)) {s2$E2} else {0})
    
  #rail options
    rails.V <- 4 * timber.rails.Section * length
    interventions.rail <- s2$DC + s2$E1 + 0.75*(if (!is.null(s2$E2)) {s2$E2} else {0})
  
  post <- mutate(materials.split[[post.Option]], bridge.Q = posts.V, interventions=interventions.post)
  rail <- mutate(materials.split[[rail.Option]], bridge.Q = rails.V, interventions = interventions.rail)
  
  LCA.matrix <- rbind(post, rail)
  
  LCA.matrix <- mutate(LCA.matrix, TotalMaterials.Q = quantities * bridge.Q / 1000,
                       materials.LC = TotalMaterials.Q * interventions,
                       Energy.LC = materials.LC * energy,
                       CO2.LC = materials.LC * CO2 * 1000,
                       NOx.LC = materials.LC * NOx * 1000,
                       SO2.LC = materials.LC * SO2 * 1000)
  LCA.results <- list(Energy = sum(LCA.matrix$Energy.LC),
                      CO2 = sum(LCA.matrix$CO2.LC),
                      NOx = sum(LCA.matrix$NOx.LC),
                      SO2 = sum(LCA.matrix$SO2.LC))
  return(LCA.results) 
}

#Initialize the variables

b.length <- 30 # units: m
p.height <- 1.2 #units m
post.Options <- c("Wood", "Steel", "PRC")
rail.options <- c("Wood", "Steel")

Rail.Option <- LCA.bridgerail(b.length, p.height, post.Options[3], rail.options[1],LCI.materials, maintenance.Exterior) #Best Design Option for 2nd Assignment

#---------------------------------------------------- LCA Roof --------------------------------------------------------------

LCA.museum <- function(length, width, height, sim.Option, purlin.Option, hole.Option, materials, dist.Events) {
  
  hole.volume <- 0.5 * length * height * 0.001
  purlin.volume <- 0.5 * length * height
  sim.volume <- length * width * height
  
  materials.split <- split(materials, materials$scope)
  s <- summary(as.factor(dist.Events$event))
  s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors=F, row.names = 1:length(s))
  
  # calculate the interventions of the sim based on different materials strategies
  if(sim.Option == "M") {
    interventions.sim <- s2$DC + 0.25*s2$M1 + 0.35*s2$M2
  } else if (sim.Option == "PRC") {
    interventions.sim <- s2$DC + 0.63*s2$M1 + 0.8*s2$M2
  }
  # calculate the interventions of the purlins based on different materials strategies
  if(purlin.Option == "T") {
    interventions.purlin <- s2$DC + 0.63*s2$M1 + 0.8*s2$M2
  } else if (purlin.Option == "PRC") {
    interventions.purlin <- s2$DC + 0.97*s2$M1 + 0.8*s2$M2
  }
  # calculate the interventions of the hole based on different materials strategies
  if(hole.Option == "G") {
    interventions.hole <- s2$DC + 0.67*s2$M1 + 0.8*s2$M2
  } else if (hole.Option == "H") {
    interventions.hole <- s2$DC + 0.53*s2$M1 + 0.8*s2$M2
  }
  sim <- mutate(materials.split[[sim.Option]], museum.Q = sim.volume, interventions =
                  interventions.sim)
  purlin <- mutate(materials.split[[purlin.Option]], museum.Q = purlin.volume, interventions =
                     interventions.purlin)
  hole <- mutate(materials.split[[hole.Option]], museum.Q = hole.volume, interventions =
                   interventions.hole)
  LCA.matrix <- rbind(sim, purlin, hole)
  
  LCA.matrix <- mutate(LCA.matrix, Totalmaterials.Q = quantities * museum.Q / 1000,
                       materials.LC = Totalmaterials.Q * interventions,
                       Energy.LC = materials.LC * energy,
                       CO2.LC = materials.LC * CO2 * 1000,
                       NOx.LC = materials.LC * NOx * 1000,
                       SO2.LC = materials.LC * SO2 * 1000)
  LCA.results <- list(Energy = sum(LCA.matrix$Energy.LC),
                      CO2 = sum(LCA.matrix$CO2.LC),
                      NOx = sum(LCA.matrix$NOx.LC),
                      SO2 = sum(LCA.matrix$SO2.LC))
  return(LCA.results)
}

#Initialize the variables

b.length <- 29.2/15 # units: m
b.width <- 1 #units m
b.height <- 1 #units m

sim.Options <- c("M", "PRC")
purlin.options <- c("T", "PRC")
hole.options <- c("G", "H")
Roof.Option <- LCA.museum(b.length, b.width, b.height, sim.Options[1], purlin.options[2], hole.options[2], LCI.materials, maintenance.Kindergarten) #Best Design Option for 2nd Assignment

#---------------------------------------------------- LCA Main Pipe --------------------------------------------------------------

#Parameter for Pipe
thickness <- 0.0123 #pipe wall thickness in meter 
out.diameter <- 0.4 #outside diameter of pipe in meter
ins.diameter <- out.diameter - 2 * thickness
cross.section <- pi * (out.diameter^2 - ins.diameter^2) * 0.25


LCA.pipe <- function(length, pipe.Option, materials, dist.Events) {
  materials.split <- split(materials, materials$material)
  s <- summary(as.factor(dist.Events$event))
  s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors=F, row.names = 1:length(s))
  # calculate the volume of material for every pipe  
  
  pipe.volume <- length * cross.section #pipe volume in cubic meters
  interventions.pipe <- s2$DC + s2$E1 + (if (!is.null(s2$E2)) {s2$E2} else {0})
  
  pipe <- mutate(materials.split[[pipe.Option]],  pipe.Q = pipe.volume, interventions = 
                   interventions.pipe)
  LCA.matrix <- rbind(pipe)
  LCA.matrix <- mutate(LCA.matrix, TotalMaterials.Q = quantities * pipe.Q, 
                       materials.LC = TotalMaterials.Q * interventions, 
                       Energy.LC = materials.LC * energy, 
                       CO2.LC = materials.LC * CO2, 
                       NOx.LC = materials.LC * NOx,
                       SO2.LC = materials.LC * SO2)
  LCA.results <- list(Energy = sum(LCA.matrix$Energy.LC), 
                      CO2 = sum(LCA.matrix$CO2.LC),
                      NOx = sum(LCA.matrix$NOx.LC),
                      SO2 = sum(LCA.matrix$SO2.LC))
  return(LCA.results)  
} 

#Initialize the variables
length <- 1000
pipe.Options <- c("PVC", "RC", "Steel")

Pipe.Option <- LCA.pipe(length, pipe.Options[1], LCI.materials, maintenance.Exterior) #Best Design Option for 2nd Assignment


#---------------------------------------------------- LCA Integrated System ------------------------------------------------------

integrated.Design <- as.data.frame(list(Energy = Foundation.Option$Energy + Frame.Option$Energy+ Rail.Option$Energy+ Roof.Option$Energy+ Pipe.Option$Energy,
                                        CO2 = Foundation.Option$CO2 + Frame.Option$CO2+ Rail.Option$CO2+ Roof.Option$CO2+ Pipe.Option$CO2,
                                        NOx = Foundation.Option$NOx + Frame.Option$NOx+ Rail.Option$NOx+ Roof.Option$NOx+ Pipe.Option$NOx,
                                        SO2 = Foundation.Option$SO2 + Frame.Option$SO2+ Rail.Option$SO2+ Roof.Option$SO2+ Pipe.Option$SO2))
Energy.costs <- 0.128
CO2.unitcost <- 26 # per metric tone
NOx.unitCost <- 42 # per metric tone
SO2.unitCosts <- 85 # per metric tone

integrated.Design <- mutate(integrated.Design,
                            Costs = (Energy * Energy.costs +
                                       CO2 * CO2.unitcost +
                                       NOx * NOx.unitCost +
                                       SO2 * SO2.unitCosts)/10^9)


# ---------------------------------------------- Part III - Multi-Objective Optimization ------------------------------------------

fitness <- function(x) {
  
  z <- numeric(7)
  x <- round(x, 0)
  y <- expand.grid(M1 = x[1], M2 = x[2], M3 = x[3], E1 = x[4], E2 = x[5], E3 = x[6])
  
  dur.ev1 <- unlist(y[1:3] / 2)
  dur.ev2 <- unlist(y[4:6] / 2)
  
  dist.1 <- apply(y[1:3], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date)
  dist.2 <- apply(y[4:6], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date)
  results <- combine.lifeTimelines(dist.1[[1]], dist.2[[1]], dur.ev1, dur.ev2)
  
  z[1] <- sum(results[["duration"]])
  z[2] <- -getmode(abs(results$frequency[1:(length(results$frequency) - 1)] - results$frequency[2:length(results$frequency)]))
  
  #---------Parameters--------- 
  #Foundation
  Depth.pile<-36 
  Depth.footing = x[12]
  Depth.raft<-2.4
  Depth.FCC<-1.2
  Factor.pile.rcc<-0.025
  Factor.footing.rcc<-0.03
  Factor.footing.FCC<-0.1
  #Structural Frame
  timber.frame.length <- 4 #length in m for squared timber profile
  timber.frame.height <- 0.5 #h = width/25
  #Rail
  PRC.post.Section <- 0.12
  timber.rails.Section<-0.08 #squared meters
  #Roof
  #hole.volume <- 0.5 * length * height * 0.001
  #purlin.volume <- 0.5 * length * height
  #sim.volume <- length * width * height
  #Pipe
  thickness <- 0.0123 #pipe wall thickness in meter 
  out.diameter <- 0.4 #outside diameter of pipe in meter
  ins.diameter <- out.diameter - 2 * thickness
  cross.section <- pi * (out.diameter^2 - ins.diameter^2) * 0.25
  
  Energy.costs <- 0.128
  CO2.unitcost <- 26 # per metric tone
  NOx.unitCost <- 42 # per metric tone
  SO2.unitCosts <- 85 # per metric tone
  
  product.1.Foundation <- LCA.Foundation(length,width,design.Op = "1. Isolated Footing")
  
  product.2.Frame <- LCA.kindergarten(c.depth, b.length, k.length, frame.Options[1], LCI.materials) 
  
  product.3.Rail <- LCA.bridgerail(b.length, p.height, post.Options[3], rail.options[1],LCI.materials, dist.2[[1]])
  
  product.4.Roof <- LCA.museum(b.length, b.width, b.height, sim.Options[1], purlin.options[2], hole.options[2], LCI.materials, dist.1[[1]])
  
  product.5.Pipe <- LCA.pipe(length, pipe.Options[1], LCI.materials, dist.2[[1]])
  
  integrated.system <- as.data.frame(list(Energy = product.1.Foundation$Energy + product.2.Frame$Energy + product.3.Rail$Energy + product.4.Roof$Energy + product.5.Pipe$Energy,
                                          CO2 = product.1.Foundation$CO2 + product.2.Frame$CO2 + product.3.Rail$CO2 + product.4.Roof$CO2 + product.5.Pipe$CO2,
                                          NOx = product.1.Foundation$NOx + product.2.Frame$NOx + product.3.Rail$NOx + product.4.Roof$NOx + product.5.Pipe$NOx,
                                          SO2 = product.1.Foundation$SO2 + product.2.Frame$SO2 + product.3.Rail$SO2 + product.4.Roof$SO2 + product.5.Pipe$SO2))
  integrated.system <- mutate(integrated.system,
                              Costs = (Energy * Energy.costs +
                                         CO2*CO2.unitcost +
                                         NOx * NOx.unitCost +
                                         SO2 * SO2.unitCosts)/10^9)
  
  z[3] <- integrated.system$Energy
  z[4] <- integrated.system$CO2
  z[5] <- integrated.system$NOx
  z[6] <- integrated.system$SO2
  z[7] <- integrated.system$Costs
  
  
  return(z) 
}

r2 <- nsga2(fitness, idim = 7, odim = 7,
            generations=10, popsize=100,
            lower.bounds=c(10,3,20,12,5,27,15),
            upper.bounds= c(20,8,35,25,10,37,23))
r2$value
r2$par
r2$pareto.optimal
which(r2$pareto.optimal==TRUE)

r2Results <- as.data.frame(r2$value)
outNames <- c("duration", "interv.dist", "energy", "co2", "nox", "so2", "cost" )
colnames(r2Results) <- outNames

pareto3 <- as.data.frame(paretoFront(r2))
colnames(pareto3) <- outNames

ggplot(r2Results, aes(x = duration, y = cost)) + geom_point(shape = 21) +
  geom_point(data = pareto3, size = 3, color="red") +
  geom_line(data = pareto3, color="blue")

input.params <- round(r2$par, 0)
all.results <- cbind(input.params, r2Results, r2$pareto.optimal)
colnames(all.results) <- c('M1','M2','M3','E1',
                           'E2','E3','Depth_F',
                           "duration", "interv.dist",
                           "energy", "co2", "nox", "so2", "cost",
                           "pareto")
par(mfrow = c(1,1))
parcoord(all.results[, 1:14], var.label = T,
         col = ifelse(all.results$pareto == TRUE, "indianred", "skyblue2"),
         lty = ifelse(all.results$pareto == TRUE, 1, 3),
         lwd = ifelse(all.results$pareto == TRUE, 3, 1))


